﻿namespace Projekt_Calcan_Conze;

using System.Net.Mail;

using Projekt_Calcan_Conze.DTOs;
using Projekt_Calcan_Conze.Models;
using Microsoft.VisualBasic;

internal static class Import
{
    public static (List<Customer> Customers, List<string> Protocol) For(string filePath)
    {
        var (customerDtos, protocol) = Import.ReadCustomerDtos(filePath);
        var customers = Import.ParseCustomers(customerDtos, protocol);
        return (customers, protocol);
    }

    private static (List<CustomerDto> CustomerDtos, List<string> Protocol) ReadCustomerDtos(
        string filePath)
    {
        List<string> protocol = [];
        List<CustomerDto> customerDtos = [];
        List<string> lines;

        try
        {
            lines =
                File.ReadAllLines(filePath)
                    .Where(line => !string.IsNullOrWhiteSpace(line)) // remove empty lines
                    .ToList();
        }
        catch (Exception)
        {
            Console.WriteLine("The file does not exist or is currently used by another program.");
            return (customerDtos, protocol);
        }

        CustomerDto? currentCustomer = null;

        foreach (string line in lines)
        {
            var lineParts =
                line.Split(Constants.Separator)
                    .Select(linePart => linePart.Trim())
                    .Where(linePart => !string.IsNullOrWhiteSpace(linePart)) // remove empty parts
                    .ToList(); // remove whitespace

            string title = lineParts[0];

            if (title is Constants.FemaleAttributeIdentifier or Constants.MaleAttributeIdentifier)
            {
                if (currentCustomer is not null)
                {
                    customerDtos.Add(currentCustomer);
                }

                currentCustomer = new CustomerDto([new CustomerAttributeDto(lineParts)]);
            }
            else if (currentCustomer is not null)
            {
                currentCustomer.Attributes.Add(new CustomerAttributeDto(lineParts));
            }
            else
            {
                protocol.Add(line);
            }
        }

        if (currentCustomer is not null)
        {
            customerDtos.Add(currentCustomer);
        }

        return (customerDtos, protocol);
    }

    private static List<Customer> ParseCustomers(List<CustomerDto> customerDtos,
                                                 List<string> protocol)
    {
        List<Customer> customers = [];

        foreach (CustomerDto customerDto in customerDtos)
        {
            Customer? customer = Import.ParseCustomer(customerDto);

            if (customer is not null)
            {
                customers.Add(customer);
            }
            else
            {
                Import.LogInvalidCustomer(protocol, customerDto);
            }
        }

        return customers;
    }

    private static Customer? ParseCustomer(CustomerDto customer)
    {
        CustomerAttributeDto firstAttribute = customer.Attributes[0];

        if (firstAttribute.Parts.Count != 4)
        {
            return null;
        }

        string title = firstAttribute.Parts[0];
        string fistName = firstAttribute.Parts[1];
        string lastName = firstAttribute.Parts[2];

        DateOnly? dateOfBirth =
            DateOnly.TryParseExact(firstAttribute.Parts[3], Constants.DateOfBirthFormat,
                                   out DateOnly date)
                ? date
                : null;

        if (dateOfBirth is null)
        {
            return null;
        }

        string? email = null;
        Address? address = null;
        List<PhoneNumber> phoneNumbers = [];

        foreach (CustomerAttributeDto attribute in customer.Attributes.Skip(1))
        {
            var attributeParts = attribute.Parts;
            string identifier = attributeParts[0];

            switch (identifier)
            {
                case Constants.AddressAttributeIdentifier:
                    if (attributeParts.Count == 4)
                    {
                        string postalCode = attributeParts[2];

                        if (postalCode.Length == 5)
                        {
                            address =
                                new Address(
                                    streetAndHouseNumber: attributeParts[1],
                                    postalCode: postalCode,
                                    city: attributeParts[3]);
                        }
                        else
                        {
                            return null;
                        }
                    }

                    break;

                case Constants.EmailAttributeIdentifier:
                    if (attributeParts.Count == 2)
                    {
                        try
                        {
                            email = attributeParts[1];

                            _ = new MailAddress(
                                email); // validate the format of the email address
                        }
                        catch (Exception)
                        {
                            return null;
                        }
                    }
                    else
                    {
                        return null;
                    }

                    break;

                case Constants.PhoneNumberAttributeIdentifier:
                    if (attributeParts.Count == 3)
                    {
                        string areaCode = attributeParts[1];
                        string number = attributeParts[2];

                        if (areaCode.Length is >= 3 and <= 5
                            && number.Length is >= 4 and <= 10)
                        {
                            phoneNumbers.Add(new PhoneNumber(areaCode, number));
                        }
                    }
                    else
                    {
                        return null;
                    }

                    break;

                default: return null;
            }
        }

        return new Customer(
            title: title,
            firstName: fistName,
            lastName: lastName,
            dateOfBirth: dateOfBirth.Value,
            email: email,
            address: address,
            phoneNumbers: phoneNumbers);
    }

    private static void LogInvalidCustomer(List<string> protocol, CustomerDto customerDto)
    {
        protocol.AddRange(
            customerDto
                .Attributes
                .Select(attribute => string.Join(Constants.Separator, attribute.Parts)));
    }
}